@@ -196,6 +196,24 @@ construct.
 Examples:
 
     .. literalinclude:: /../../tests/yamlparser/fixtures/jinja-string01.yaml
+
+The tag ``!j2-yaml:`` is similar to the ``!j2:`` tag, just that it loads the
+Jinja-rendered string as YAML and embeds it in the calling YAML construct. This
+provides a very flexible and convenient way of generating pieces of YAML
+structures. One of use cases is defining complex YAML structures with much
+simpler configuration, without any duplication.
+
+Examples:
+
+    .. literalinclude:: /../../tests/yamlparser/fixtures/jinja-yaml01.yaml
+
+Another use case is controlling lists dynamically, like conditionally adding
+list elements based on project configuration.
+
+Examples:
+
+    .. literalinclude:: /../../tests/yamlparser/fixtures/jinja-yaml02.yaml
+
 """
 
 import functools
@@ -203,6 +221,7 @@ import io
 import logging
 import os
 import re
+import copy
 
 import jinja2
 import yaml
@@ -368,6 +387,14 @@ class BaseYAMLObject(YAMLObject):
     yaml_dumper = LocalDumper
 
 
+class J2Yaml(BaseYAMLObject):
+    yaml_tag = u"!j2-yaml:"
+
+    @classmethod
+    def from_yaml(cls, loader, node):
+        return Jinja2YamlLoader(node.value, loader.search_path)
+
+
 class J2String(BaseYAMLObject):
     yaml_tag = u"!j2:"
 
@@ -571,6 +598,26 @@ class Jinja2Loader(CustomLoader):
         self._template.environment.loader = self._loader
         return self._template.render(kwargs)
 
+    def get_object_to_format(self):
+        return self
+
+
+class LateYamlLoader(CustomLoader):
+    """A loader for data rendered via Jinja2, to be loaded as YAML and then deep formatted."""
+
+    def __init__(self, yaml_str, loader):
+        self._yaml_str = yaml_str
+        self._loader = loader
+
+    def get_object_to_format(self):
+        return load(self._yaml_str, search_path=self._loader._search_path)
+
+
+class Jinja2YamlLoader(Jinja2Loader):
+    def format(self, **kwargs):
+        yaml_str = super(Jinja2YamlLoader, self).format(**kwargs)
+        return LateYamlLoader(yaml_str, self)
+
 
 class CustomLoaderCollection(object):
     """Helper class to format a collection of CustomLoader objects"""
@@ -598,6 +645,11 @@ class LazyLoader(CustomLoader):
     def __repr__(self):
         return "%s %s" % (self._cls.yaml_tag, self._node.value)
 
+    def __deepcopy__(self, memodict={}):
+        return LazyLoader(
+            (copy.deepcopy(self._cls), self._loader, copy.deepcopy(self._node))
+        )
+
     def format(self, *args, **kwargs):
         node = yaml.ScalarNode(
             tag=self._node.tag, value=self._node.value.format(*args, **kwargs)
