@@ -12,22 +12,25 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from functools import wraps
 import logging
+import sys
 
 import xml.etree.ElementTree as XML
 
 from jenkins_jobs.errors import InvalidAttributeError
 from jenkins_jobs.errors import JenkinsJobsException
 from jenkins_jobs.errors import MissingAttributeError
+from jenkins_jobs.modules import hudson_model
+
+import pkg_resources
 
 
 def build_trends_publisher(plugin_name, xml_element, data):
-    """Helper to create various trend publishers.
-    """
+    """Helper to create various trend publishers."""
 
     def append_thresholds(element, data, only_totals):
-        """Appends the status thresholds.
-        """
+        """Appends the status thresholds."""
 
         for status in ["unstable", "failed"]:
             status_data = data.get(status, {})
@@ -510,7 +513,10 @@ def trigger_get_parameter_order(registry, plugin):
     return None
 
 
-def trigger_project(tconfigs, project_def, param_order=None):
+def trigger_project(tconfigs, project_def, registry, param_order=None):
+
+    info = registry.get_plugin_info("parameterized-trigger")
+    plugin_version = pkg_resources.parse_version(info.get("version", str(sys.maxsize)))
 
     logger = logging.getLogger("%s:trigger_project" % __name__)
     pt_prefix = "hudson.plugins.parameterizedtrigger."
@@ -547,6 +553,12 @@ def trigger_project(tconfigs, project_def, param_order=None):
                 ("property-file", "propertiesFile", None),
                 ("fail-on-missing", "failTriggerOnMissing", False),
             ]
+
+            if plugin_version >= pkg_resources.parse_version("2.35.2"):
+                property_file_mapping.append(
+                    ("property-multiline", "textParamValueOnNewLine", False)
+                )
+
             convert_mapping_to_xml(
                 params, project_def, property_file_mapping, fail_required=True
             )
@@ -613,6 +625,26 @@ def trigger_project(tconfigs, project_def, param_order=None):
                 )
 
 
+def trigger_threshold(
+    parent_element, element_name, threshold_name, supported_thresholds=None
+):
+    """Generate a resultThreshold XML element for build/join triggers"""
+    element = XML.SubElement(parent_element, element_name)
+
+    try:
+        threshold = hudson_model.THRESHOLDS[threshold_name.upper()]
+    except KeyError:
+        if not supported_thresholds:
+            supported_thresholds = hudson_model.THRESHOLDS.keys()
+        raise JenkinsJobsException(
+            "threshold must be one of %s" % ", ".join(supported_thresholds)
+        )
+    XML.SubElement(element, "name").text = threshold["name"]
+    XML.SubElement(element, "ordinal").text = threshold["ordinal"]
+    XML.SubElement(element, "color").text = threshold["color"]
+    return element
+
+
 def convert_mapping_to_xml(parent, data, mapping, fail_required=True):
     """Convert mapping to XML
 
@@ -695,3 +727,23 @@ def jms_messaging_common(parent, subelement, data):
         ("msg-content", "messageContent", ""),
     ]
     convert_mapping_to_xml(namespace, data, mapping, fail_required=True)
+
+
+def check_mutual_exclusive_data_args(data_arg_position, *args):
+    mutual_exclusive_args = set(args)
+
+    def validator(f):
+        @wraps(f)
+        def wrap(*args):
+            actual_args = set(args[data_arg_position].keys())
+            if len(actual_args & mutual_exclusive_args) > 1:
+                raise JenkinsJobsException(
+                    "Args: {} in {} are mutual exclusive. Please define one of it.".format(
+                        mutual_exclusive_args, f.__name__
+                    )
+                )
+            return f(*args)
+
+        return wrap
+
+    return validator
