@@ -61,6 +61,8 @@ def string_param(registry, xml_parent, data):
     :arg str name: the name of the parameter
     :arg str default: the default value of the parameter (optional)
     :arg str description: a description of the parameter (optional)
+    :arg bool trim: strip whitespaces from the begnning and end
+        of the string (optional, default: false)
 
     Example::
 
@@ -69,17 +71,22 @@ def string_param(registry, xml_parent, data):
             name: FOO
             default: bar
             description: "A parameter named FOO, defaults to 'bar'."
+            trim: true
     """
-    base_param(
+    pdef = base_param(
         registry, xml_parent, data, True, "hudson.model.StringParameterDefinition"
     )
 
+    mapping = [("trim", "trim", False)]
+    helpers.convert_mapping_to_xml(pdef, data, mapping, fail_required=True)
+
 
 def promoted_param(registry, xml_parent, data):
     """yaml: promoted build
     A promoted build parameter.
-    Requires the Jenkins :jenkins-wiki:`Promoted Builds Plugin
-    <Promoted+Builds+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Promoted Builds Plugin
+    <promoted-builds>`.
 
     :arg str name: the name of the parameter (required)
     :arg str project-name: the job from which the user can pick runs (required)
@@ -338,9 +345,9 @@ def choice_param(registry, xml_parent, data):
 
 def credentials_param(registry, xml_parent, data):
     """yaml: credentials
-    A credentials selection parameter. Requires the Jenkins
-    :jenkins-wiki:`Credentials Plugin
-    <Credentials+Plugin>`.
+    A credentials selection parameter.
+
+    Requires the Jenkins :jenkins-plugins:`Credentials Plugin <credentials>`.
 
     :arg str name: the name of the parameter
     :arg str type: credential type (optional, default 'any')
@@ -354,7 +361,7 @@ def credentials_param(registry, xml_parent, data):
 
     :arg bool required: whether this parameter is required (optional, default
         false)
-    :arg string default: default credentials ID (optional)
+    :arg str default: default credentials ID (optional)
     :arg str description: a description of the parameter (optional)
 
     Example:
@@ -415,11 +422,26 @@ def run_param(registry, xml_parent, data):
     helpers.convert_mapping_to_xml(pdef, data, mapping, fail_required=True)
 
 
+@helpers.check_mutual_exclusive_data_args(
+    2, "value", "property-file", "groovy-script", "groovy-script-file"
+)
+@helpers.check_mutual_exclusive_data_args(
+    2, "default-value", "default-property-file", "default-groovy-script"
+)
+@helpers.check_mutual_exclusive_data_args(
+    2,
+    "value-description",
+    "description-property-file",
+    "description-groovy-script",
+    "description-groovy-script-file",
+)
+@helpers.check_mutual_exclusive_data_args(2, "javascript", "javascript-file")
 def extended_choice_param(registry, xml_parent, data):
     """yaml: extended-choice
     Creates an extended choice parameter where values can be read from a file
-    Requires the Jenkins :jenkins-wiki:`Extended Choice Parameter Plugin
-    <Extended+Choice+Parameter+plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Extended Choice Parameter Plugin
+    <extended-choice-parameter>`.
 
     :arg str name: name of the parameter
     :arg str description: description of the parameter
@@ -432,7 +454,8 @@ def extended_choice_param(registry, xml_parent, data):
     :arg str visible-items: number of items to show in the list
         (optional, default 5)
     :arg str type: type of select, can be single-select, multi-select,
-        radio, checkbox or textbox (optional, default single-select)
+        multi-level-single-select, multi-level-multi-select,
+        radio, checkbox, textbox, json (optional, default single-select)
     :arg str value: comma separated list of values for the single select
         or multi-select box (optional, default '')
     :arg str default-value: used to set the initial selection of the
@@ -462,9 +485,15 @@ def extended_choice_param(registry, xml_parent, data):
         groovy script (optional, default '')
     :arg str description-groovy-script: location of groovy script when value
         description needs to come from a groovy script (optional, default '')
+    :arg str description-groovy-script-file: location of groovy script file when value
+        description needs to come from a groovy script (optional, default '')
     :arg str description-groovy-classpath: classpath for the value description
         groovy script (optional, default '')
-
+    :arg str javascript: the javascript script contents (optional, default '')
+    :arg str javascript-file: location of javasript script file to generate
+        parameters (optional, default '')
+    :arg bool save-json-parameter-to-file: if json parameter should be saved
+        to file (optional, default False)
 
     Minimal Example:
 
@@ -491,14 +520,20 @@ def extended_choice_param(registry, xml_parent, data):
     choicedict = {
         "single-select": "PT_SINGLE_SELECT",
         "multi-select": "PT_MULTI_SELECT",
+        "multi-level-single-select": "PT_MULTI_LEVEL_SINGLE_SELECT",
+        "multi-level-multi-select": "PT_MULTI_LEVEL_MULTI_SELECT",
         "radio": "PT_RADIO",
         "checkbox": "PT_CHECKBOX",
         "textbox": "PT_TEXTBOX",
+        "json": "PT_JSON",
         "PT_SINGLE_SELECT": "PT_SINGLE_SELECT",
         "PT_MULTI_SELECT": "PT_MULTI_SELECT",
+        "PT_MULTI_LEVEL_SINGLE_SELECT": "PT_MULTI_LEVEL_SINGLE_SELECT",
+        "PT_MULTI_LEVEL_MULTI_SELECT": "PT_MULTI_LEVEL_MULTI_SELECT",
         "PT_RADIO": "PT_RADIO",
         "PT_CHECKBOX": "PT_CHECKBOX",
         "PT_TEXTBOX": "PT_TEXTBOX",
+        "PT_JSON": "PT_JSON",
     }
     mapping = [
         ("value", "value", ""),
@@ -521,7 +556,11 @@ def extended_choice_param(registry, xml_parent, data):
         ("default-groovy-script", "defaultGroovyScript", ""),
         ("default-groovy-classpath", "defaultGroovyClasspath", ""),
         ("description-groovy-script", "descriptionGroovyScript", ""),
+        ("description-groovy-script-file", "descriptionGroovyScriptFile", ""),
         ("description-groovy-classpath", "descriptionGroovyClasspath", ""),
+        ("javascript", "javascript", ""),
+        ("javascript-file", "javascriptFile", ""),
+        ("save-json-parameter-to-file", "saveJSONParameterToFile", False),
     ]
     helpers.convert_mapping_to_xml(pdef, data, mapping, fail_required=True)
 
@@ -529,8 +568,9 @@ def extended_choice_param(registry, xml_parent, data):
 def validating_string_param(registry, xml_parent, data):
     """yaml: validating-string
     A validating string parameter
-    Requires the Jenkins :jenkins-wiki:`Validating String Plugin
-    <Validating+String+Parameter+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Validating String Plugin
+    <validating-string-parameter>`.
 
     :arg str name: the name of the parameter
     :arg str default: the default value of the parameter (optional)
@@ -563,8 +603,9 @@ def validating_string_param(registry, xml_parent, data):
 def svn_tags_param(registry, xml_parent, data):
     """yaml: svn-tags
     A svn tag parameter
-    Requires the Jenkins :jenkins-wiki:`Parameterized Trigger Plugin
-    <Parameterized+Trigger+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Parameterized Trigger Plugin
+    <parameterized-trigger>`.
 
     :arg str name: the name of the parameter
     :arg str url: the url to list tags from
@@ -585,7 +626,7 @@ def svn_tags_param(registry, xml_parent, data):
             name: BRANCH_NAME
             default: release
             description: A parameter named BRANCH_NAME default is release
-            url: http://svn.example.com/repo
+            url: http://svn.example.org/repo
             filter: [A-za-z0-9]*
     """
     pdef = base_param(
@@ -610,6 +651,7 @@ def svn_tags_param(registry, xml_parent, data):
 def dynamic_choice_param(registry, xml_parent, data):
     """yaml: dynamic-choice
     Dynamic Choice Parameter
+
     Requires the Jenkins :jenkins-wiki:`Jenkins Dynamic Parameter Plug-in
     <Dynamic+Parameter+Plug-in>`.
 
@@ -638,6 +680,7 @@ def dynamic_choice_param(registry, xml_parent, data):
 def dynamic_string_param(registry, xml_parent, data):
     """yaml: dynamic-string
     Dynamic Parameter
+
     Requires the Jenkins :jenkins-wiki:`Jenkins Dynamic Parameter Plug-in
     <Dynamic+Parameter+Plug-in>`.
 
@@ -666,6 +709,7 @@ def dynamic_string_param(registry, xml_parent, data):
 def dynamic_choice_scriptler_param(registry, xml_parent, data):
     """yaml: dynamic-choice-scriptler
     Dynamic Choice Parameter (Scriptler)
+
     Requires the Jenkins :jenkins-wiki:`Jenkins Dynamic Parameter Plug-in
     <Dynamic+Parameter+Plug-in>`.
 
@@ -704,6 +748,7 @@ def dynamic_choice_scriptler_param(registry, xml_parent, data):
 def dynamic_string_scriptler_param(registry, xml_parent, data):
     """yaml: dynamic-string-scriptler
     Dynamic Parameter (Scriptler)
+
     Requires the Jenkins :jenkins-wiki:`Jenkins Dynamic Parameter Plug-in
     <Dynamic+Parameter+Plug-in>`.
 
@@ -799,8 +844,9 @@ def dynamic_scriptler_param_common(registry, xml_parent, data, ptype):
 def matrix_combinations_param(registry, xml_parent, data):
     """yaml: matrix-combinations
     Matrix combinations parameter
-    Requires the Jenkins :jenkins-wiki:`Matrix Combinations Plugin
-    <Matrix+Combinations+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Matrix Combinations Plugin
+    <matrix-combinations-parameter>`.
 
     :arg str name: the name of the parameter
     :arg str description: a description of the parameter (optional)
@@ -834,8 +880,9 @@ def copyartifact_build_selector_param(registry, xml_parent, data):
     """yaml: copyartifact-build-selector
 
     Control via a build parameter, which build the copyartifact plugin should
-    copy when it is configured to use 'build-param'. Requires the Jenkins
-    :jenkins-wiki:`Copy Artifact plugin <Copy+Artifact+Plugin>`.
+    copy when it is configured to use 'build-param'.
+
+    Requires the Jenkins :jenkins-plugins:`Copy Artifact plugin <copyartifact>`.
 
     :arg str name: name of the build parameter to store the selection in
     :arg str description: a description of the parameter (optional)
@@ -865,8 +912,9 @@ def maven_metadata_param(registry, xml_parent, data):
     """yaml: maven-metadata
     This parameter allows the resolution of maven artifact versions
     by contacting the repository and reading the maven-metadata.xml.
-    Requires the Jenkins :jenkins-wiki:`Maven Metadata Plugin
-    <Maven+Metadata+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Maven Metadata Plugin
+    <maven-metadata-plugin>`.
 
     :arg str name: Name of the parameter
     :arg str description: Description of the parameter (optional)
@@ -934,8 +982,9 @@ def maven_metadata_param(registry, xml_parent, data):
 def hidden_param(parser, xml_parent, data):
     """yaml: hidden
     Allows you to use parameters hidden from the build with parameter page.
-    Requires the Jenkins :jenkins-wiki:`Hidden Parameter Plugin
-    <Hidden+Parameter+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Hidden Parameter Plugin
+    <hidden-parameter>`.
 
     :arg str name: the name of the parameter
     :arg str default: the default value of the parameter (optional)
@@ -957,8 +1006,9 @@ def random_string_param(registry, xml_parent, data):
     """yaml: random-string
     This parameter generates a random string and passes it to the
     build, preventing Jenkins from combining queued builds.
-    Requires the Jenkins :jenkins-wiki:`Random String Parameter Plugin
-    <Random+String+Parameter+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Random String Parameter Plugin
+    <random-string-parameter>`.
 
     :arg str name: Name of the parameter
     :arg str description: Description of the parameter (default '')
@@ -990,8 +1040,9 @@ def git_parameter_param(registry, xml_parent, data):
     """yaml: git-parameter
     This parameter allows you to select a git tag, branch or revision number as
     parameter in Parametrized builds.
-    Requires the Jenkins :jenkins-wiki:`Git Parameter Plugin
-    <Git+Parameter+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Git Parameter Plugin
+    <git-parameter>`.
 
     :arg str name: Name of the parameter
     :arg str description: Description of the parameter (default '')
@@ -1093,6 +1144,222 @@ def git_parameter_param(registry, xml_parent, data):
     helpers.convert_mapping_to_xml(pdef, data, mapping, fail_required=True)
 
 
+def param_separator(registry, xml_parent, data):
+    """yaml: parameter-separator
+    A parameter separator.
+
+    :arg str name: name of the separator (default "", the plugin will assign a randomly generated UUID if not specified)
+    :arg str separator-style: the style of the separator. Uses CSS. (default "")
+    :arg str section-header-text: the section header text of the separator (default "")
+    :arg str section-header-style: the section header style (CSS) of the separator. Uses CSS. (default "")
+
+    Example::
+
+      parameters:
+        - parameter-separator:
+            name: lorem
+            separator-style: FOO
+            section-header: bar
+            section-header-style: font-weight:bold;z-index:10000
+    """
+    element_name = "jenkins.plugins.parameter__separator.ParameterSeparatorDefinition"
+    pdef = XML.SubElement(xml_parent, element_name)
+    XML.SubElement(pdef, "description")  # To create empty description tag
+    mapping = [
+        ("name", "name", ""),
+        ("section-header", "sectionHeader", ""),
+        ("section-header-style", "sectionHeaderStyle", ""),
+        ("separator-style", "separatorStyle", ""),
+    ]
+    helpers.convert_mapping_to_xml(pdef, data, mapping, fail_required=True)
+
+
+def __handle_unochoice_script(data, pdef, script_type, main_script_xml):
+
+    if script_type == "script":
+        secure_script_xml = XML.SubElement(main_script_xml, "secureScript")
+    elif script_type == "fallback-script":
+        secure_script_xml = XML.SubElement(main_script_xml, "secureFallbackScript")
+
+    sub_script_xml = XML.SubElement(secure_script_xml, "script")
+    groovy_sandbox_xml = XML.SubElement(secure_script_xml, "sandbox")
+
+    script = data.get(script_type, {})
+    sub_script_xml.text = script.get("groovy", "")
+    groovy_sandbox_xml.text = str(script.get("use-groovy-sandbox", True)).lower()
+
+    if "script-additional-classpath" in script:
+        classpath_xml = XML.SubElement(secure_script_xml, "classpath")
+        for additional_classpath in script.get("script-additional-classpath"):
+            entry_xml = XML.SubElement(classpath_xml, "entry")
+            url_xml = XML.SubElement(entry_xml, "url")
+            url_xml.text = additional_classpath
+
+
+def active_choices_param(registry, xml_parent, data):
+    """yaml: active-choices
+    Active Choices Parameter
+
+    Requires the Jenkins :jenkins-wiki:`Active Choices Plug-in
+    <active+choices+plugin>`.
+
+    :arg str name: Name of the parameter (required).
+    :arg str description: Description of the parameter.
+    :arg list script: Use a Groovy script to define the parameter.
+
+        :Parameter: * **groovy** (`str`) Groovy DSL Script
+                    * **use-groovy-sandbox** (`bool`) To run this
+                        Groovy script in a sandbox with limited abilities
+                        (default True)
+                    * **script-additional-classpath** (`list`) Additional
+                        classpath entries accessible from the script.
+    :arg list fallback-script: Use a Fallback script. If the script
+        (specified above) fails, the fallback script will be used as a fallback.
+
+        :Parameter: * **groovy** (`str`) Groovy DSL Script
+                    * **use-groovy-sandbox** (`bool`) To run this Groovy
+                        script in a sandbox with limited abilities.
+                        (default True)
+                    * **script-additional-classpath** (`list`) Additional
+                        classpath entries accessible from the script.
+    :arg bool enable-filters: If enabled a text box will appear next to
+        this element and will permit the user to filter its entries. The
+        list values never get re-evaluated (default False).
+    :arg int filter-starts-at: How many characters a user must enter
+        before the filter is applied (default 1).
+    :arg str choice-type: type of the choices. (default 'single-select')
+
+        :Allowed Values: * **single-select**
+                    * **multi-select**
+                    * **radio-buttons**
+                    * **checkboxes**
+
+    Minimal Example:
+
+    .. literalinclude::
+        /../../tests/yamlparser/fixtures/active-choices-param001.yaml
+       :language: yaml
+
+    Full Example:
+
+    .. literalinclude::
+        /../../tests/yamlparser/fixtures/active-choices-param002.yaml
+       :language: yaml
+    """
+    element_name = "org.biouno.unochoice.ChoiceParameter"
+    pdef = XML.SubElement(xml_parent, element_name)
+
+    valid_choice_types_dict = {
+        "single-select": "PT_SINGLE_SELECT",
+        "multi-select": "PT_MULTI_SELECT",
+        "radio-buttons": "PT_RADIO",
+        "checkboxes": "PT_CHECKBOX",
+    }
+
+    mapping = [
+        ("name", "name", None),
+        ("description", "description", ""),
+        ("choice-type", "choiceType", "single-select", valid_choice_types_dict),
+        ("enable-filters", "filterable", False),
+        ("filter-starts-at", "filterLength", 1),
+        ("_project-name", "projectName", None),
+        ("_project-full-name", "projectFullName", None),
+    ]
+
+    main_script_xml = XML.SubElement(pdef, "script")
+    main_script_xml.set("class", "org.biouno.unochoice.model.GroovyScript")
+    __handle_unochoice_script(data, pdef, "fallback-script", main_script_xml)
+    __handle_unochoice_script(data, pdef, "script", main_script_xml)
+
+    helpers.convert_mapping_to_xml(pdef, data, mapping, fail_required=True)
+
+
+def dynamic_reference_param(registry, xml_parent, data):
+    """yaml: dynamic-reference
+    Active Choices Reactive Reference Parameter
+
+    Requires the Jenkins :jenkins-wiki:`Active Choices Plug-in
+    <active+choices+plugin>`.
+
+    :arg str name: Name of the parameter (required).
+    :arg str description: Description of the parameter.
+    :arg list script: Use a Groovy script to define the parameter.
+
+        :Parameter: * **groovy** (`str`) Groovy DSL Script
+                    * **use-groovy-sandbox** (`bool`) To run this
+                        Groovy script in a sandbox with limited abilities
+                        (default True)
+                    * **script-additional-classpath** (`list`) Additional
+                        classpath entries accessible from the script.
+    :arg list fallback-script: Use a Fallback script. If the script
+        (specified above) fails, the fallback script will be used as a fallback.
+
+        :Parameter: * **groovy** (`str`) Groovy DSL Script
+                    * **use-groovy-sandbox** (`bool`) To run this Groovy
+                        script in a sandbox with limited abilities.
+                        (default True)
+                    * **script-additional-classpath** (`list`) Additional
+                        classpath entries accessible from the script.
+    :arg bool omit-value-field: By default Dynamic Reference Parameters always
+        include a hidden input for the value. If your script creates an input
+        HTML element, you can check this option and the value input field will
+        be omitted (default False).
+    :arg str referenced-parameters: Comma separated list of other job
+         parameters referenced in the uno-choice script. When any of the
+         referenced parameters are updated, the Groovy script will
+         re-evaluate the choice list using the updated values of referenced
+         parameters.
+    :arg str choice-type: type of the choices. (default 'input-text-box')
+
+        :Allowed Values: * **input-text-box**
+                    * **numbered-list**
+                    * **bullet-items-list**
+                    * **formatted-html**
+                    * **formatted-hidden-html**
+
+    Minimal Example:
+
+    .. literalinclude::
+        /../../tests/yamlparser/fixtures/dynamic-reference-param001.yaml
+       :language: yaml
+
+    Full Example:
+
+    .. literalinclude::
+        /../../tests/yamlparser/fixtures/dynamic-reference-param002.yaml
+       :language: yaml
+    """
+    element_name = "org.biouno.unochoice.DynamicReferenceParameter"
+    pdef = XML.SubElement(xml_parent, element_name)
+
+    valid_choice_types_dict = {
+        "input-text-box": "ET_TEXT_BOX",
+        "numbered-list": "ET_ORDERED_LIST",
+        "bullet-items-list": "ET_UNORDERED_LIST",
+        "formatted-html": "ET_FORMATTED_HTML",
+        "formatted-hidden-html": "ET_FORMATTED_HIDDEN_HTML",
+    }
+
+    mapping = [
+        ("name", "name", None),
+        ("description", "description", ""),
+        ("choice-type", "choiceType", "input-text-box", valid_choice_types_dict),
+        ("_project-name", "projectName", None),
+        ("_project-full-name", "projectFullName", None),
+        ("referenced-parameters", "referencedParameters", ""),
+        ("omit-value-field", "omitValueField", False),
+    ]
+
+    main_script_xml = XML.SubElement(pdef, "script")
+    main_script_xml.set("class", "org.biouno.unochoice.model.GroovyScript")
+    __handle_unochoice_script(data, pdef, "fallback-script", main_script_xml)
+    __handle_unochoice_script(data, pdef, "script", main_script_xml)
+
+    XML.SubElement(pdef, "parameters")  # Empty parameters tag
+
+    helpers.convert_mapping_to_xml(pdef, data, mapping, fail_required=True)
+
+
 class Parameters(jenkins_jobs.modules.base.Base):
     sequence = 21
 
@@ -1121,4 +1388,9 @@ class Parameters(jenkins_jobs.modules.base.Base):
             if pdefs is None:
                 pdefs = XML.SubElement(pdefp, "parameterDefinitions")
             for param in parameters:
+                # Pass job name to the uno-choice plugin
+                param_type = next(iter(param))
+                if param_type in ("active-choices", "dynamic-reference"):
+                    param[param_type]["_project-name"] = data["name"].split("/")[-1]
+                    param[param_type]["_project-full-name"] = data["name"]
                 self.registry.dispatch("parameter", pdefs, param)
