@@ -26,11 +26,12 @@ which means that there is no guarantee that its behavior (or configuration)
 will not change, even between minor releases.
 
 Plugins required:
-    * :jenkins-wiki:`Workflow Plugin <Workflow+Plugin>`.
-    * :jenkins-wiki:`Pipeline Multibranch Defaults Plugin
-      <Pipeline+Multibranch+Defaults+Plugin>` (optional)
-    * :jenkins-wiki:`Basic Branch Build Strategies Plugin
-      <Basic+Branch+Build+Strategies+Plugin>` (optional)
+
+* :jenkins-plugins:`Workflow Plugin <workflow-aggregator>`.
+* :jenkins-plugins:`Pipeline Multibranch Defaults Plugin
+  <pipeline-multibranch-defaults>` (optional)
+* :jenkins-plugins:`Basic Branch Build Strategies Plugin
+  <basic-branch-build-strategies>` (optional)
 
 :Job Parameters:
 
@@ -59,10 +60,18 @@ Plugins required:
       (default '-1, forever')
     * **script-path** (`str`): Path to Jenkinsfile, relative to workspace.
       (default 'Jenkinsfile')
+    * **script-id** (`str`): Script id from the global Jenkins script store
+      provided by the config-file provider plugin. Mutually exclusive with
+      **script-path** option.
+    * **sandbox** (`bool`): This option is strongly recommended if the
+      Jenkinsfile is using load to evaluate a groovy source file from an
+      SCM repository. Usable only with **script-id** option. (default 'false')
 
 Job examples:
 
-.. literalinclude:: /../../tests/multibranch/fixtures/multibranch_defaults.yaml
+.. literalinclude:: /../../tests/multibranch/fixtures/multibranch_defaults_id_mode.yaml
+
+.. literalinclude:: /../../tests/multibranch/fixtures/multibranch_defaults_path_mode.yaml
 
 .. literalinclude:: /../../tests/multibranch/fixtures/multi_scm_full.yaml
 
@@ -77,6 +86,7 @@ import six
 from jenkins_jobs.modules.scm import git_extensions
 from jenkins_jobs.errors import InvalidAttributeError
 from jenkins_jobs.errors import JenkinsJobsException
+from jenkins_jobs.xml_config import remove_ignorable_whitespace
 
 logger = logging.getLogger(str(__name__))
 
@@ -84,8 +94,39 @@ logger = logging.getLogger(str(__name__))
 class WorkflowMultiBranch(jenkins_jobs.modules.base.Base):
     sequence = 0
     multibranch_path = "org.jenkinsci.plugins.workflow.multibranch"
+    multibranch_defaults_path = "org.jenkinsci.pipeline.workflow.multibranch"
     jenkins_class = "".join([multibranch_path, ".WorkflowMultiBranchProject"])
-    jenkins_factory_class = "".join([multibranch_path, ".WorkflowBranchProjectFactory"])
+    jenkins_factory = {
+        "script_path": {
+            "class": "".join([multibranch_path, ".WorkflowBranchProjectFactory"])
+        },
+        "script_id": {
+            "class": "".join(
+                [
+                    multibranch_defaults_path,
+                    ".defaults.PipelineBranchDefaultsProjectFactory",
+                ]
+            ),
+            "plugin": "pipeline-multibranch-defaults",
+        },
+    }
+
+    @staticmethod
+    def _factory_opts_check(data):
+
+        sandbox = data.get("sandbox", None)
+        script_id = data.get("script-id", None)
+        script_path = data.get("script-path", None)
+
+        if script_id and script_path:
+            error_msg = "script-id and script-path are mutually exclusive options"
+            raise JenkinsJobsException(error_msg)
+        elif not script_id and sandbox:
+            error_msg = (
+                "Sandbox mode is applicable only for multibranch with defaults"
+                "project type used with script-id option"
+            )
+            raise JenkinsJobsException(error_msg)
 
     def root_xml(self, data):
         xml_parent = XML.Element(self.jenkins_class)
@@ -267,35 +308,57 @@ class WorkflowMultiBranch(jenkins_jobs.modules.base.Base):
         # Factory #
         ###########
 
-        factory = XML.SubElement(
-            xml_parent, "factory", {"class": self.jenkins_factory_class}
-        )
+        self._factory_opts_check(data)
+
+        if data.get("script-id"):
+            mode = "script_id"
+            fopts_map = (
+                ("script-id", "scriptId", None),
+                ("sandbox", "useSandbox", None),
+            )
+        else:
+            mode = "script_path"
+            fopts_map = (("script-path", "scriptPath", "Jenkinsfile"),)
+
+        factory = XML.SubElement(xml_parent, "factory", self.jenkins_factory[mode])
         XML.SubElement(
             factory, "owner", {"class": self.jenkins_class, "reference": "../.."}
         )
-        XML.SubElement(factory, "scriptPath").text = data.get(
-            "script-path", "Jenkinsfile"
-        )
+
+        # multibranch default
+
+        helpers.convert_mapping_to_xml(factory, data, fopts_map, fail_required=False)
 
         return xml_parent
 
 
 class WorkflowMultiBranchDefaults(WorkflowMultiBranch):
-    jenkins_class = (
-        "org.jenkinsci.plugins.pipeline.multibranch"
-        ".defaults.PipelineMultiBranchDefaultsProject"
-    )
-    jenkins_factory_class = (
-        "org.jenkinsci.plugins.pipeline.multibranch"
-        ".defaults.PipelineBranchDefaultsProjectFactory"
+    multibranch_path = "org.jenkinsci.plugins.workflow.multibranch"
+    multibranch_defaults_path = "org.jenkinsci.plugins.pipeline.multibranch"
+    jenkins_class = "".join(
+        [multibranch_defaults_path, ".defaults.PipelineMultiBranchDefaultsProject"]
     )
+    jenkins_factory = {
+        "script_path": {
+            "class": "".join([multibranch_path, ".WorkflowBranchProjectFactory"]),
+            "plugin": "workflow-multibranch",
+        },
+        "script_id": {
+            "class": "".join(
+                [
+                    multibranch_defaults_path,
+                    ".defaults.PipelineBranchDefaultsProjectFactory",
+                ]
+            )
+        },
+    }
 
 
 def bitbucket_scm(xml_parent, data):
     r"""Configure BitBucket scm
 
-    Requires the :jenkins-wiki:`Bitbucket Branch Source Plugin
-    <Bitbucket+Branch+Source+Plugin>`.
+    Requires the :jenkins-plugins:`Bitbucket Branch Source Plugin
+    <cloudbees-bitbucket-branch-source>`.
 
     :arg str credentials-id: The credential to use to scan BitBucket.
         (required)
@@ -305,10 +368,16 @@ def bitbucket_scm(xml_parent, data):
 
     :arg bool discover-tags: Discovers tags on the repository.
         (default false)
+    :arg bool lfs: Git LFS pull after checkout.
+        (default false)
     :arg str server-url: The address of the bitbucket server. (optional)
     :arg str head-filter-regex: A regular expression for filtering
-        discovered source branches. Requires the :jenkins-wiki:`SCM API Plugin
-        <SCM+API+Plugin>`.
+        discovered source branches. Requires the :jenkins-plugins:`SCM API
+        Plugin <scm-api>`.
+    :arg list head-pr-filter-behaviors: Definition of Filter Branch PR behaviors.
+        Requires the :jenkins-plugins:`SCM Filter Branch PR Plugin
+        <scm-filter-branch-pr>`.
+        Refer to :func:`~add_filter_branch_pr_behaviors <add_filter_branch_pr_behaviors>`.
     :arg str discover-branch: Discovers branches on the repository.
         Valid options: ex-pr, only-pr, all.
         Value is not specified by default.
@@ -338,14 +407,15 @@ def bitbucket_scm(xml_parent, data):
         be checked out to a local branch named master, and a remote branch
         origin/develop/new-feature will be checked out to a local branch
         named develop/newfeature.
-        Requires the :jenkins-wiki:`Git Plugin <Git+Plugin>`.
+        Requires the :jenkins-plugins:`Git Plugin <git>`.
+    :arg list(str) refspecs: Which refspecs to look for.
     :arg dict checkout-over-ssh: Checkout repo over ssh.
 
         * **credentials** ('str'): Credentials to use for
             checkout of the repo over ssh.
 
     :arg dict filter-by-name-wildcard: Enable filter by name with wildcards.
-        Requires the :jenkins-wiki:`SCM API Plugin <SCM+API+Plugin>`.
+        Requires the :jenkins-plugins:`SCM API Plugin <scm-api>`.
 
         * **includes** ('str'): Space-separated list
             of name patterns to consider. You may use * as a wildcard;
@@ -418,14 +488,47 @@ def bitbucket_scm(xml_parent, data):
     helpers.convert_mapping_to_xml(source, data, mapping_optional, fail_required=False)
 
     traits = XML.SubElement(source, "traits")
+
+    if data.get("refspecs"):
+        refspec_trait = XML.SubElement(
+            traits,
+            "jenkins.plugins.git.traits.RefSpecsSCMSourceTrait",
+            {"plugin": "git"},
+        )
+        templates = XML.SubElement(refspec_trait, "templates")
+        refspecs = data.get("refspecs")
+        for refspec in refspecs:
+            e = XML.SubElement(
+                templates,
+                (
+                    "jenkins.plugins.git.traits"
+                    ".RefSpecsSCMSourceTrait_-RefSpecTemplate"
+                ),
+            )
+            XML.SubElement(e, "value").text = refspec
+
     if data.get("discover-tags", False):
         XML.SubElement(
             traits, "com.cloudbees.jenkins.plugins.bitbucket.TagDiscoveryTrait"
         )
+
+    if data.get("lfs", False):
+        gitlfspull = XML.SubElement(
+            traits, "jenkins.plugins.git.traits.GitLFSPullTrait", {"plugin": "git"}
+        )
+        XML.SubElement(
+            gitlfspull,
+            "extension",
+            {"class": "hudson.plugins.git.extensions.impl.GitLFSPull"},
+        )
+
     if data.get("head-filter-regex", None):
         rshf = XML.SubElement(traits, "jenkins.scm.impl.trait.RegexSCMHeadFilterTrait")
         XML.SubElement(rshf, "regex").text = data.get("head-filter-regex")
 
+    if data.get("head-pr-filter-behaviors", None):
+        add_filter_branch_pr_behaviors(traits, data.get("head-pr-filter-behaviors"))
+
     if data.get("discover-pr-origin", None):
         dpro = XML.SubElement(
             traits,
@@ -546,8 +649,8 @@ def bitbucket_scm(xml_parent, data):
 def gerrit_scm(xml_parent, data):
     """Configure Gerrit SCM
 
-    Requires the :jenkins-wiki:`Gerrit Code Review Plugin
-    <Gerrit+Code+Review+Plugin>`.
+    Requires the :jenkins-plugins:`Gerrit Code Review Plugin
+    <gerrit-code-review>`.
 
     :arg str url: The git url. (required)
     :arg str credentials-id: The credential to use to connect to the GIT URL.
@@ -569,6 +672,13 @@ def gerrit_scm(xml_parent, data):
         (like to disable SCM triggering or to override the pipeline durability)
         (optional)
         Refer to :func:`~property_strategies <property_strategies>`.
+    :arg dict filter-checks: Enable the filtering by pending checks, allowing to
+        discover the changes that need validation only. This feature is using
+        the gerrit checks plugin.
+        (optional)
+        query-operator: Name of the query operator, supported values are:
+        'SCHEME' or 'ID'.
+        query-string: Value of the query operator.
 
     Minimal Example:
 
@@ -583,7 +693,10 @@ def gerrit_scm(xml_parent, data):
     source = XML.SubElement(
         xml_parent,
         "source",
-        {"class": "jenkins.plugins.gerrit.GerritSCMSource", "plugin": "gerrit"},
+        {
+            "class": "jenkins.plugins.gerrit.GerritSCMSource",
+            "plugin": "gerrit-code-review",
+        },
     )
     source_mapping = [
         ("", "id", "-".join(["gr", data.get("url", "")])),
@@ -632,11 +745,25 @@ def gerrit_scm(xml_parent, data):
     if data.get("build-strategies", None):
         build_strategies(xml_parent, data)
 
+    # FilterChecks Trait
+    checks = data.get("filter-checks", None)
+    if checks:
+        checks_trait = XML.SubElement(
+            traits, "jenkins.plugins.gerrit.traits.FilterChecksTrait"
+        )
+        checks_source_mapping = [
+            ("query-operator", "queryOperator", None),
+            ("query-string", "queryString", None),
+        ]
+        helpers.convert_mapping_to_xml(
+            checks_trait, checks, checks_source_mapping, fail_required=True
+        )
+
 
 def git_scm(xml_parent, data):
     r"""Configure Git SCM
 
-    Requires the :jenkins-wiki:`Git Plugin <Git+Plugin>`.
+    Requires the :jenkins-plugins:`Git Plugin <git>`.
 
     :arg str url: The git repo url. (required)
     :arg str credentials-id: The credential to use to connect to the GIT repo.
@@ -649,8 +776,12 @@ def git_scm(xml_parent, data):
     :arg bool ignore-on-push-notifications: If a job should not trigger upon
         push notifications. (default false)
     :arg str head-filter-regex: A regular expression for filtering
-        discovered source branches. Requires the :jenkins-wiki:`SCM API Plugin
-        <SCM+API+Plugin>`.
+        discovered source branches. Requires the :jenkins-plugins:`SCM API
+        Plugin <scm-api>`.
+    :arg list head-pr-filter-behaviors: Definition of Filter Branch PR behaviors.
+        Requires the :jenkins-plugins:`SCM Filter Branch PR Plugin
+        <scm-filter-branch-pr>`.
+        Refer to :func:`~add_filter_branch_pr_behaviors <add_filter_branch_pr_behaviors>`.
     :arg list build-strategies: Provides control over whether to build a branch
         (or branch like things such as change requests and tags) whenever it is
         discovered initially or a change from the previous revision has been
@@ -732,6 +863,9 @@ def git_scm(xml_parent, data):
         rshf = XML.SubElement(traits, "jenkins.scm.impl.trait.RegexSCMHeadFilterTrait")
         XML.SubElement(rshf, "regex").text = data.get("head-filter-regex")
 
+    if data.get("head-pr-filter-behaviors", None):
+        add_filter_branch_pr_behaviors(traits, data.get("head-pr-filter-behaviors"))
+
     if data.get("property-strategies", None):
         property_strategies(xml_parent, data)
 
@@ -753,8 +887,8 @@ def git_scm(xml_parent, data):
 def github_scm(xml_parent, data):
     r"""Configure GitHub SCM
 
-    Requires the :jenkins-wiki:`GitHub Branch Source Plugin
-    <GitHub+Branch+Source+Plugin>`.
+    Requires the :jenkins-plugins:`GitHub Branch Source Plugin
+    <github-branch-source>`.
 
     :arg str api-uri: The GitHub API uri for hosted / on-site GitHub. Must
         first be configured in Global Configuration. (default GitHub)
@@ -779,9 +913,13 @@ def github_scm(xml_parent, data):
         (default 'contributors')
     :arg str discover-pr-origin: Discovers pull requests where the origin
         repository is the same as the target repository.
-        Valid options: merge-current, current, both.  (default 'merge-current')
+        Valid options: merge-current, current, both, false.  (default 'merge-current')
     :arg bool discover-tags: Discovers tags on the repository.
         (default false)
+    :arg list head-pr-filter-behaviors: Definition of Filter Branch PR behaviors.
+        Requires the :jenkins-plugins:`SCM Filter Branch PR Plugin
+        <scm-filter-branch-pr>`.
+        Refer to :func:`~add_filter_branch_pr_behaviors <add_filter_branch_pr_behaviors>`.
     :arg list build-strategies: Provides control over whether to build a branch
         (or branch like things such as change requests and tags) whenever it is
         discovered initially or a change from the previous revision has been
@@ -789,8 +927,8 @@ def github_scm(xml_parent, data):
         Refer to :func:`~build_strategies <build_strategies>`.
     :arg str notification-context: Change the default GitHub check notification
         context from "continuous-integration/jenkins/SUFFIX" to a custom text,
-        Requires the :jenkins-wiki:`Github Custom Notification Context SCM
-        Behaviour <Github+Custom+Notification+Context+SCM+Behaviour+Plugin>`.
+        Requires the :jenkins-plugins:`Github Custom Notification Context SCM
+        Behaviour <github-scm-trait-notification-context>`.
     :arg dict property-strategies: Provides control over how to build a branch
         (like to disable SCM triggering or to override the pipeline durability)
         (optional)
@@ -810,6 +948,7 @@ def github_scm(xml_parent, data):
             notifications on pull requests (default false) (Requires the
             :jenkins-plugins:`GitHub Branch Source Plugin
             <disable-github-multibranch-status>`.)
+        * **refspecs** (`list(str)`): Which refspecs to fetch.
         * **submodule** (`dict`)
             * **disable** (`bool`) - By disabling support for submodules you
               can still keep using basic git plugin functionality and just have
@@ -921,23 +1060,27 @@ def github_scm(xml_parent, data):
         XML.SubElement(dprf, "trust").attrib["class"] = trust_map[trust]
 
     dpro_strategy = data.get("discover-pr-origin", "merge-current")
-    dpro = XML.SubElement(
-        traits, "".join([github_path_dscore, ".OriginPullRequestDiscoveryTrait"])
-    )
-    dpro_strategy_map = {"merge-current": "1", "current": "2", "both": "3"}
-    if dpro_strategy not in dpro_strategy_map:
-        raise InvalidAttributeError(
-            "discover-pr-origin", dpro_strategy, dpro_strategy_map.keys()
+    if dpro_strategy:
+        dpro = XML.SubElement(
+            traits, "".join([github_path_dscore, ".OriginPullRequestDiscoveryTrait"])
         )
-    dpro_mapping = [
-        ("discover-pr-origin", "strategyId", "merge-current", dpro_strategy_map)
-    ]
-    helpers.convert_mapping_to_xml(dpro, data, dpro_mapping, fail_required=True)
+        dpro_strategy_map = {"merge-current": "1", "current": "2", "both": "3"}
+        if dpro_strategy not in dpro_strategy_map:
+            raise InvalidAttributeError(
+                "discover-pr-origin", dpro_strategy, dpro_strategy_map.keys()
+            )
+        dpro_mapping = [
+            ("discover-pr-origin", "strategyId", "merge-current", dpro_strategy_map)
+        ]
+        helpers.convert_mapping_to_xml(dpro, data, dpro_mapping, fail_required=True)
 
     if data.get("head-filter-regex", None):
         rshf = XML.SubElement(traits, "jenkins.scm.impl.trait.RegexSCMHeadFilterTrait")
         XML.SubElement(rshf, "regex").text = data.get("head-filter-regex")
 
+    if data.get("head-pr-filter-behaviors", None):
+        add_filter_branch_pr_behaviors(traits, data.get("head-pr-filter-behaviors"))
+
     if data.get("property-strategies", None):
         property_strategies(xml_parent, data)
 
@@ -964,6 +1107,24 @@ def github_scm(xml_parent, data):
     # - use-author
     git_extensions(traits, data)
 
+    if data.get("refspecs"):
+        refspec_trait = XML.SubElement(
+            traits,
+            "jenkins.plugins.git.traits.RefSpecsSCMSourceTrait",
+            {"plugin": "git"},
+        )
+        templates = XML.SubElement(refspec_trait, "templates")
+        refspecs = data.get("refspecs")
+        for refspec in refspecs:
+            e = XML.SubElement(
+                templates,
+                (
+                    "jenkins.plugins.git.traits"
+                    ".RefSpecsSCMSourceTrait_-RefSpecTemplate"
+                ),
+            )
+            XML.SubElement(e, "value").text = refspec
+
     # github-only extensions
     disable_github_status_path_dscore = (
         "com.adobe.jenkins.disable__github__multibranch__status"
@@ -979,11 +1140,19 @@ def github_scm(xml_parent, data):
 def build_strategies(xml_parent, data):
     """Configure Basic Branch Build Strategies.
 
-    Requires the :jenkins-wiki:`Basic Branch Build Strategies Plugin
-    <Basic+Branch+Build+Strategies+Plugin>`.
+    Requires the :jenkins-plugins:`Basic Branch Build Strategies Plugin
+    <basic-branch-build-strategies>`.
+
+    Other build strategies can be configured via raw XML injection.
 
     :arg list build-strategies: Definition of build strategies.
 
+        * **all-strategies-match** (dict): All sub strategies must match for
+            this strategy to match.
+            * **strategies** (list): Sub strategies
+        * **any-strategies-match** (dict): Builds whenever any of the sub
+            strategies match.
+            * **strategies** (list): Sub strategies
         * **tags** (dict): Builds tags
             * **ignore-tags-newer-than** (int) The number of days since the tag
                 was created before it is eligible for automatic building.
@@ -1022,12 +1191,35 @@ def build_strategies(xml_parent, data):
                     for example: `master release*` (default `*`)
                 * **excludes** (str) Name patterns to ignore even if matched
                     by the includes list. For example: release (optional)
+        * **raw** (dict): Injects raw BuildStrategy XML to use other build
+            strategy plugins.
 
     """
 
     basic_build_strategies = "jenkins.branch.buildstrategies.basic"
-    bbs = XML.SubElement(xml_parent, "buildStrategies")
-    for bbs_list in data.get("build-strategies", None):
+    if data.get("build-strategies", None):
+        bbs_data = data.get("build-strategies", None)
+        bbs = XML.SubElement(xml_parent, "buildStrategies")
+    else:
+        bbs_data = data.get("strategies", None)
+        bbs = XML.SubElement(xml_parent, "strategies")
+    for bbs_list in bbs_data:
+        if "all-strategies-match" in bbs_list:
+            all_elem = XML.SubElement(
+                bbs,
+                "".join([basic_build_strategies, ".AllBranchBuildStrategyImpl"]),
+                {"plugin": "basic-branch-build-strategies"},
+            )
+            build_strategies(all_elem, bbs_list["all-strategies-match"])
+
+        if "any-strategies-match" in bbs_list:
+            any_elem = XML.SubElement(
+                bbs,
+                "".join([basic_build_strategies, ".AnyBranchBuildStrategyImpl"]),
+                {"plugin": "basic-branch-build-strategies"},
+            )
+            build_strategies(any_elem, bbs_list["any-strategies-match"])
+
         if "tags" in bbs_list:
             tags = bbs_list["tags"]
             tags_elem = XML.SubElement(
@@ -1148,11 +1340,16 @@ def build_strategies(xml_parent, data):
                         fail_required=False,
                     )
 
+        if "raw" in bbs_list:
+            raw_xml = XML.fromstring(bbs_list["raw"].get("xml"))
+            remove_ignorable_whitespace(raw_xml)
+            XML.SubElement(bbs, None).append(raw_xml)
+
 
 def property_strategies(xml_parent, data):
     """Configure Basic Branch Property Strategies.
 
-    Requires the :jenkins-wiki:`Branch API Plugin <Branch+API+Plugin>`.
+    Requires the :jenkins-plugins:`Branch API Plugin <branch-api>`.
 
     :arg dict property-strategies: Definition of property strategies.  Either
         `named-branches` or `all-branches` may be specified, but not both.
@@ -1165,9 +1362,30 @@ def property_strategies(xml_parent, data):
             * **pipeline-branch-durability-override** (str): Set a custom
                 branch speed/durability level. Valid values:
                 performance-optimized, survivable-nonatomic, or
-                max-survivability (optional) Requires the :jenkins-wiki:
-                `Pipeline Multibranch Plugin <Pipeline+Multibranch+Plugin>`
-
+                max-survivability (optional)
+                Requires the :jenkins-plugins:`Pipeline Multibranch Plugin
+                <workflow-multibranch>`
+            * **trigger-build-on-pr-comment** (str): The comment body to
+                trigger a new build for a PR job when it is received. This
+                is compiled as a case insensitive regular expression, so
+                use ``".*"`` to trigger a build on any comment whatsoever.
+                (optional)
+                Requires the :jenkins-plugins:`GitHub PR Comment Build Plugin
+                <github-pr-comment-build>`
+            * **trigger-build-on-pr-review** (bool): This property will
+                cause a job for a pull request ``(PR-*)`` to be triggered
+                immediately when a review is made on the PR in GitHub.
+                This has no effect on jobs that are not for pull requests.
+                (optional)
+                Requires the :jenkins-plugins:`GitHub PR Comment Build Plugin
+                <github-pr-comment-build>`
+            * **trigger-build-on-pr-update** (bool): This property will
+                cause a job for a pull request ``(PR-*)`` to be triggered
+                immediately when the PR title or description is edited in
+                GitHub. This has no effect on jobs that are not for pull
+                requests. (optional)
+                Requires the :jenkins-plugins:`GitHub PR Comment Build Plugin
+                <github-pr-comment-build>`
         * **named-branches** (dict): Named branches get different properties.
             Comprised of a list of defaults and a list of property strategy
             exceptions for use with specific branches.
@@ -1181,8 +1399,30 @@ def property_strategies(xml_parent, data):
                 * **pipeline-branch-durability-override** (str): Set a custom
                     branch speed/durability level. Valid values:
                     performance-optimized, survivable-nonatomic, or
-                    max-survivability (optional) Requires the :jenkins-wiki:
-                    `Pipeline Multibranch Plugin <Pipeline+Multibranch+Plugin>`
+                    max-survivability (optional)
+                    Requires the :jenkins-plugins:`Pipeline Multibranch Plugin
+                    <workflow-multibranch>`
+                * **trigger-build-on-pr-comment** (str): The comment body to
+                    trigger a new build for a PR job when it is received. This
+                    is compiled as a case insensitive regular expression, so
+                    use ``".*"`` to trigger a build on any comment whatsoever.
+                    (optional)
+                    Requires the :jenkins-plugins:`GitHub PR Comment Build Plugin
+                    <github-pr-comment-build>`
+                * **trigger-build-on-pr-review** (bool): This property will
+                    cause a job for a pull request ``(PR-*)`` to be triggered
+                    immediately when a review is made on the PR in GitHub.
+                    This has no effect on jobs that are not for pull requests.
+                    (optional)
+                    Requires the :jenkins-plugins:`GitHub PR Comment Build Plugin
+                    <github-pr-comment-build>`
+                * **trigger-build-on-pr-update** (bool): This property will
+                    cause a job for a pull request ``(PR-*)`` to be triggered
+                    immediately when the PR title or description is edited in
+                    GitHub. This has no effect on jobs that are not for pull
+                    requests. (optional)
+                    Requires the :jenkins-plugins:`GitHub PR Comment Build Plugin
+                    <github-pr-comment-build>`
 
             * **exceptions** (list): A list of branch names and the property
                 strategies to be used on that branch, instead of any listed
@@ -1199,9 +1439,9 @@ def property_strategies(xml_parent, data):
                         * **pipeline-branch-durability-override** (str): Set a
                             custom branch speed/durability level. Valid values:
                             performance-optimized, survivable-nonatomic, or
-                            max-survivability (optional) Requires the
-                            :jenkins-wiki:`Pipeline Multibranch Plugin
-                            <Pipeline+Multibranch+Plugin>`
+                            max-survivability (optional)
+                            Requires the :jenkins-plugins:`Pipeline
+                            Multibranch Plugin <workflow-multibranch>`
     """
 
     valid_prop_strats = ["all-branches", "named-branches"]
@@ -1348,6 +1588,7 @@ def apply_property_strategies(props_elem, props_list):
 
     basic_property_strategies = "jenkins.branch"
     workflow_multibranch = "org.jenkinsci.plugins.workflow.multibranch"
+    pr_comment_build = "com.adobe.jenkins.github__pr__comment__build"
     # Valid options for the pipeline branch durability override.
     pbdo_map = collections.OrderedDict(
         [
@@ -1357,6 +1598,13 @@ def apply_property_strategies(props_elem, props_list):
         ]
     )
 
+    pcb_bool_opts = collections.OrderedDict(
+        [
+            ("trigger-build-on-pr-review", ".TriggerPRReviewBranchProperty"),
+            ("trigger-build-on-pr-update", ".TriggerPRUpdateBranchProperty"),
+        ]
+    )
+
     for dbs_list in props_list:
 
         if dbs_list.get("suppress-scm-triggering", False):
@@ -1377,3 +1625,129 @@ def apply_property_strategies(props_elem, props_list):
                 {"plugin": "workflow-multibranch"},
             )
             XML.SubElement(pbdo_elem, "hint").text = pbdo_map.get(pbdo_val)
+
+        tbopc_val = dbs_list.get("trigger-build-on-pr-comment", None)
+        if tbopc_val:
+            tbopc_elem = XML.SubElement(
+                props_elem,
+                "".join([pr_comment_build, ".TriggerPRCommentBranchProperty"]),
+                {"plugin": "github-pr-comment-build"},
+            )
+            XML.SubElement(tbopc_elem, "commentBody").text = tbopc_val
+        for opt in pcb_bool_opts:
+            if dbs_list.get(opt, False):
+                XML.SubElement(
+                    props_elem,
+                    "".join([pr_comment_build, pcb_bool_opts.get(opt)]),
+                    {"plugin": "github-pr-comment-build"},
+                )
+
+
+def add_filter_branch_pr_behaviors(traits, data):
+    """Configure Filter Branch PR behaviors
+
+    Requires the :jenkins-plugins:`SCM Filter Branch PR Plugin
+    <scm-filter-branch-pr>`.
+
+    :arg list head-pr-filter-behaviors: Definition of filters.
+
+        * **head-pr-destined-regex** (dict): Filter by name incl. PR destined to
+            this branch with regexp
+
+            * **branch-regexp** (str) Regular expression to filter branches and
+                PRs (optional, default ".*")
+            * **tag-regexp** (str) Regular expression to filter tags
+                (optional, default "(?!.*)")
+
+        * **head-pr-destined-wildcard** (dict): Filter by name incl. PR
+            destined to this branch with wildcard
+
+            * **branch-includes** (str) Wildcard expression to include branches
+                and PRs (optional, default "*")
+            * **tag-includes** (str) Wildcard expression to include tags
+                (optional, default "")
+            * **branch-excludes** (str) Wildcard expression to exclude branches
+                and PRs (optional, default "")
+            * **tag-excludes** (str) Wildcard expression to exclude tags
+                (optional, default "*")
+
+        * **head-pr-originated-regex** (dict): Filter by name incl. PR destined
+            to this branch with regexp
+
+            * **branch-regexp** (str) Regular expression to filter branches
+                and PRs (optional, default ".*")
+            * **tag-regexp** (str) Regular expression to filter tags
+                (optional, default "(?!.*)")
+
+        * **head-pr-originated-wildcard** (dict): Filter by name incl. PR
+            destined to this branch with wildcard
+
+            * **branch-includes** (str) Wildcard expression to include branches
+                and PRs (optional, default "*")
+            * **tag-includes** (str) Wildcard expression to include tags
+                (optional, default "")
+            * **branch-excludes** (str) Wildcard expression to exclude branches
+                and PRs (optional, default "")
+            * **tag-excludes** (str) Wildcard expression to exclude tags
+                (optional, default "*")
+    """
+
+    regexp_mapping = [
+        ("branch-regexp", "regex", ".*"),
+        ("tag-regexp", "tagRegex", "(?!.*)"),
+    ]
+    wildcard_mapping = [
+        ("branch-includes", "includes", "*"),
+        ("branch-excludes", "excludes", ""),
+        ("tag-includes", "tagIncludes", ""),
+        ("tag-excludes", "tagExcludes", "*"),
+    ]
+
+    if data.get("head-pr-destined-regex"):
+        rshf = XML.SubElement(
+            traits,
+            "net.gleske.scmfilter.impl.trait.RegexSCMHeadFilterTrait",
+            {"plugin": "scm-filter-branch-pr"},
+        )
+        helpers.convert_mapping_to_xml(
+            rshf, data.get("head-pr-destined-regex"), regexp_mapping, fail_required=True
+        )
+
+    if data.get("head-pr-destined-wildcard"):
+        wshf = XML.SubElement(
+            traits,
+            "net.gleske.scmfilter.impl.trait.WildcardSCMHeadFilterTrait",
+            {"plugin": "scm-filter-branch-pr"},
+        )
+        helpers.convert_mapping_to_xml(
+            wshf,
+            data.get("head-pr-destined-wildcard"),
+            wildcard_mapping,
+            fail_required=True,
+        )
+
+    if data.get("head-pr-originated-regex"):
+        rsof = XML.SubElement(
+            traits,
+            "net.gleske.scmfilter.impl.trait.RegexSCMOriginFilterTrait",
+            {"plugin": "scm-filter-branch-pr"},
+        )
+        helpers.convert_mapping_to_xml(
+            rsof,
+            data.get("head-pr-originated-regex"),
+            regexp_mapping,
+            fail_required=True,
+        )
+
+    if data.get("head-pr-originated-wildcard"):
+        wsof = XML.SubElement(
+            traits,
+            "net.gleske.scmfilter.impl.trait.WildcardSCMOriginFilterTrait",
+            {"plugin": "scm-filter-branch-pr"},
+        )
+        helpers.convert_mapping_to_xml(
+            wsof,
+            data.get("head-pr-originated-wildcard"),
+            wildcard_mapping,
+            fail_required=True,
+        )
