@@ -1,4 +1,5 @@
 # Copyright 2012 Hewlett-Packard Development Company, L.P.
+# Copyright 2020 Liberty Global B.V.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -46,6 +47,7 @@ import jenkins_jobs.modules.helpers as helpers
 def builds_chain_fingerprinter(registry, xml_parent, data):
     """yaml: builds-chain-fingerprinter
     Builds chain fingerprinter.
+
     Requires the Jenkins :jenkins-wiki:`Builds chain fingerprinter Plugin
     <Builds+chain+fingerprinter>`.
 
@@ -75,7 +77,8 @@ def builds_chain_fingerprinter(registry, xml_parent, data):
 def ownership(registry, xml_parent, data):
     """yaml: ownership
     Plugin provides explicit ownership for jobs and slave nodes.
-    Requires the Jenkins :jenkins-wiki:`Ownership Plugin <Ownership+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Ownership Plugin <ownership>`.
 
     :arg bool enabled: whether ownership enabled (default : true)
     :arg str owner: the owner of job
@@ -107,8 +110,9 @@ def promoted_build(registry, xml_parent, data):
     name must be created via the web interface in the job in order for the job
     promotion to persist. Promotion processes themselves cannot be configured
     by jenkins-jobs due to the separate storage of plugin configuration files.
-    Requires the Jenkins :jenkins-wiki:`Promoted Builds Plugin
-    <Promoted+Builds+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Promoted Builds Plugin
+    <promoted-builds>`.
 
     :arg list names: the promoted build names (optional)
 
@@ -130,7 +134,8 @@ def promoted_build(registry, xml_parent, data):
 def gitbucket(parser, xml_parent, data):
     """yaml: gitbucket
     Integrate GitBucket to Jenkins.
-    Requires the Jenkins :jenkins-wiki:`GitBucket Plugin <GitBucket+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`GitBucket Plugin <gitbucket>`.
 
     :arg str url: GitBucket URL to issue (required)
     :arg bool link-enabled: Enable hyperlink to issue (default false)
@@ -186,7 +191,8 @@ def gitlab(registry, xml_parent, data):
     """yaml: gitlab
     Sets the GitLab connection for the project. Configured via Jenkins Global
     Configuration.
-    Requires the Jenkins :jenkins-wiki:`GitLab Plugin <GitLab+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`GitLab Plugin <gitlab-plugin>`.
 
     :arg str connection: the GitLab connection name (required)
 
@@ -206,8 +212,9 @@ def gitlab(registry, xml_parent, data):
 def gitlab_logo(registry, xml_parent, data):
     """yaml: gitlab-logo
     Configures the GitLab-Logo Plugin.
-    Requires the Jenkins :jenkins-wiki:`GitLab Logo Plugin
-    <GitLab+Logo+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`GitLab Logo Plugin
+    <gitlab-logo>`.
 
     :arg str repository-name: the GitLab repository name (required)
 
@@ -223,10 +230,56 @@ def gitlab_logo(registry, xml_parent, data):
     helpers.convert_mapping_to_xml(logo, data, mapping, fail_required=True)
 
 
+def gogs(registry, xml_parent, data):
+    """yaml: gogs
+    Sets the Gogs webhook properties for the project.
+
+    Requires the Jenkins :jenkins-plugins:`Gogs Plugin <gogs-webhook>`.
+
+    :arg str secret: webhook secret (default '')
+    :arg str branch-filter: filter which needs to match to trigger a job (default '')
+
+    Minimal Example:
+
+    .. literalinclude:: /../../tests/properties/fixtures/gogs-minimal.yaml
+       :language: yaml
+
+    Full Example:
+
+    .. literalinclude:: /../../tests/properties/fixtures/gogs-full.yaml
+       :language: yaml
+    """
+    gogs = XML.SubElement(xml_parent, "org.jenkinsci.plugins.gogs.GogsProjectProperty")
+    mapping = [("branch-filter", "gogsBranchFilter", ""), ("secret", "gogsSecret", "")]
+    helpers.convert_mapping_to_xml(gogs, data, mapping)
+
+
+def naginator_opt_out(registry, xml_parent, data):
+    """yaml: naginator-opt-out
+    Lets you opt-out so no rebuild option for Naginator is added.
+
+    Requires the Jenkins :jenkins-plugins:`Naginator Plugin <naginator>`.
+
+    :arg bool opt-out: disables the rebuild option if True (default False).
+
+    Example:
+
+    .. literalinclude:: /../../tests/properties/fixtures/naginator-opt-out002.yaml
+       :language: yaml
+    """
+
+    opt_out = XML.SubElement(
+        xml_parent, "com.chikli.hudson.plugin.naginator." "NaginatorOptOutProperty"
+    )
+    mapping = [("opt-out", "optOut", False)]
+    helpers.convert_mapping_to_xml(opt_out, data, mapping, fail_required=True)
+
+
 def disk_usage(registry, xml_parent, data):
     """yaml: disk-usage
     Enables the Disk Usage Plugin.
-    Requires the Jenkins :jenkins-wiki:`Disk Usage Plugin <Disk+Usage+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Disk Usage Plugin <disk-usage>`.
 
     Example:
 
@@ -239,7 +292,8 @@ def disk_usage(registry, xml_parent, data):
 def least_load(registry, xml_parent, data):
     """yaml: least-load
     Enables the Least Load Plugin.
-    Requires the Jenkins :jenkins-wiki:`Least Load Plugin <Least+Load+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Least Load Plugin <leastload>`.
 
     :arg bool disabled: whether or not leastload is disabled (default true)
 
@@ -259,8 +313,9 @@ def least_load(registry, xml_parent, data):
 def throttle(registry, xml_parent, data):
     """yaml: throttle
     Throttles the number of builds for this job.
-    Requires the Jenkins :jenkins-wiki:`Throttle Concurrent Builds Plugin
-    <Throttle+Concurrent+Builds+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Throttle Concurrent Builds Plugin
+    <throttle-concurrents>`.
 
     :arg str option: throttle `project` (throttle the project alone)
          or `category` (throttle the project as part of one or more categories)
@@ -320,8 +375,8 @@ def throttle(registry, xml_parent, data):
 def branch_api(registry, xml_parent, data):
     """yaml: branch-api
     Enforces a minimum time between builds based on the desired maximum rate.
-    Requires the Jenkins :jenkins-wiki:`Branch API Plugin
-    <Branch+API+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Branch API Plugin <branch-api>`.
 
     :arg int number-of-builds: The maximum number of builds allowed within
         the specified time period. (default 1)
@@ -362,8 +417,7 @@ def branch_api(registry, xml_parent, data):
 def sidebar(registry, xml_parent, data):
     """yaml: sidebar
     Allows you to add links in the sidebar.
-    Requires the Jenkins :jenkins-wiki:`Sidebar-Link Plugin
-    <Sidebar-Link+Plugin>`.
+    Requires the Jenkins :jenkins-plugins:`Sidebar-Link Plugin <sidebar-link>`.
 
     :arg str url: url to link to (optional)
     :arg str text: text for the link (optional)
@@ -390,7 +444,8 @@ def sidebar(registry, xml_parent, data):
 def inject(registry, xml_parent, data):
     """yaml: inject
     Allows you to inject environment variables into the build.
-    Requires the Jenkins :jenkins-wiki:`Env Inject Plugin <EnvInject+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`EnvInject Plugin <envinject>`.
 
     :arg str properties-file: file to read with properties (optional)
     :arg str properties-content: key=value properties (optional)
@@ -509,8 +564,8 @@ def authorization(registry, xml_parent, data):
        :language: yaml
     """
 
-    # get the folder name if it exists
-    in_a_folder = data.pop("_use_folder_perms")
+    # check if it's a folder or a job
+    is_a_folder = data.pop("_is_a_folder", None) if data else False
 
     credentials = "com.cloudbees.plugins.credentials.CredentialsProvider."
     ownership = "com.synopsys.arc.jenkins.plugins.ownership.OwnershipPlugin."
@@ -539,22 +594,18 @@ def authorization(registry, xml_parent, data):
     }
 
     if data:
-        if in_a_folder:
-            matrix = XML.SubElement(
-                xml_parent,
-                "com.cloudbees.hudson.plugins.folder.properties.AuthorizationMatrixProperty",
-            )
-            XML.SubElement(
-                matrix,
-                "inheritanceStrategy",
-                {
-                    "class": "org.jenkinsci.plugins.matrixauth.inheritance.InheritParentStrategy"
-                },
-            )
+        if is_a_folder:
+            element_name = "com.cloudbees.hudson.plugins.folder.properties.AuthorizationMatrixProperty"
         else:
-            matrix = XML.SubElement(
-                xml_parent, "hudson.security.AuthorizationMatrixProperty"
-            )
+            element_name = "hudson.security.AuthorizationMatrixProperty"
+        matrix = XML.SubElement(xml_parent, element_name)
+        XML.SubElement(
+            matrix,
+            "inheritanceStrategy",
+            {
+                "class": "org.jenkinsci.plugins.matrixauth.inheritance.InheritParentStrategy"
+            },
+        )
 
         for (username, perms) in data.items():
             for perm in perms:
@@ -569,8 +620,8 @@ def priority_sorter(registry, xml_parent, data):
     """yaml: priority-sorter
     Allows simple ordering of builds, using a configurable job priority.
 
-    Requires the Jenkins :jenkins-wiki:`Priority Sorter Plugin
-    <Priority+Sorter+Plugin>`.
+    Requires the Jenkins :jenkins-plugins:`Priority Sorter Plugin
+    <PrioritySorter>`.
 
     :arg int priority: Priority of the job.  Higher value means higher
         priority, with 3 as the default priority. (required)
@@ -615,8 +666,8 @@ def build_blocker(registry, xml_parent, data):
     if at least one name of currently running jobs
     is matching with one of the given regular expressions.
 
-    Requires the Jenkins :jenkins-wiki:`Build Blocker Plugin
-    <Build+Blocker+Plugin>`.
+    Requires the Jenkins :jenkins-plugins:`Build Blocker Plugin
+    <build-blocker-plugin>`.
 
     :arg bool use-build-blocker: Enable or disable build blocker (default true)
     :arg list blocking-jobs: One regular expression per line to select
@@ -668,8 +719,8 @@ def copyartifact(registry, xml_parent, data):
     Specify a list of projects that have access to copy the artifacts of
     this project.
 
-    Requires the Jenkins :jenkins-wiki:`Copy Artifact plugin
-    <Copy+Artifact+Plugin>`.
+    Requires the Jenkins :jenkins-plugins:`Copy Artifact plugin
+    <copyartifact>`.
 
     :arg str projects: comma separated list of projects that can copy
         artifacts of this project. Wild card character '*' is available.
@@ -704,7 +755,7 @@ def batch_tasks(registry, xml_parent, data):
     Batch tasks and builds "lock" the workspace, so when one of those
     activities is in progress, all the others will block in the queue.
 
-    Requires the Jenkins :jenkins-wiki:`Batch Task Plugin <Batch+Task+Plugin>`.
+    Requires the Jenkins :jenkins-plugins:`Batch Task Plugin <batch-task>`.
 
     :arg list batch-tasks: Batch tasks.
 
@@ -731,7 +782,7 @@ def heavy_job(registry, xml_parent, data):
     This plugin allows you to define "weight" on each job,
     and making each job consume that many executors
 
-    Requires the Jenkins :jenkins-wiki:`Heavy Job Plugin <Heavy+Job+Plugin>`.
+    Requires the Jenkins :jenkins-plugins:`Heavy Job Plugin <heavy-job>`.
 
     :arg int weight: Specify the total number of executors
         that this job should occupy (default 1)
@@ -754,8 +805,8 @@ def slave_utilization(registry, xml_parent, data):
     This plugin allows you to specify the percentage of a slave's capacity a
     job wants to use.
 
-    Requires the Jenkins :jenkins-wiki:`Slave Utilization Plugin
-    <Slave+Utilization+Plugin>`.
+    Requires the Jenkins :jenkins-plugins:`Slave Utilization Plugin
+    <slave-utilization-plugin>`.
 
     :arg int slave-percentage: Specify the percentage of a slave's execution
         slots that this job should occupy (default 0)
@@ -786,8 +837,8 @@ def slave_utilization(registry, xml_parent, data):
 
 def delivery_pipeline(registry, xml_parent, data):
     """yaml: delivery-pipeline
-    Requires the Jenkins :jenkins-wiki:`Delivery Pipeline Plugin
-    <Delivery+Pipeline+Plugin>`.
+    Requires the Jenkins :jenkins-plugins:`Delivery Pipeline Plugin
+    <delivery-pipeline-plugin>`.
 
     :arg str stage: Name of the stage for this job (default '')
     :arg str task: Name of the task for this job (default '')
@@ -823,7 +874,7 @@ def zeromq_event(registry, xml_parent, data):
     (start, complete, finish) to a ZMQ PUB socket.
 
     Requires the Jenkins `ZMQ Event Publisher.
-    <https://git.openstack.org/cgit/openstack-infra/zmq-event-publisher>`_
+    <https://opendev.org/x/zmq-event-publisher>`_
 
     Example:
 
@@ -843,7 +894,7 @@ def zeromq_event(registry, xml_parent, data):
 
 def slack(registry, xml_parent, data):
     """yaml: slack
-    Requires the Jenkins :jenkins-wiki:`Slack Plugin <Slack+Plugin>`
+    Requires the Jenkins :jenkins-plugins:`Slack Plugin <slack>`.
 
     When using Slack Plugin version < 2.0, Slack Plugin itself requires a
     publisher aswell as properties please note that you have to add the
@@ -920,7 +971,8 @@ def rebuild(registry, xml_parent, data):
     This plug-in allows the user to rebuild a parameterized build without
     entering the parameters again.It will also allow the user to edit the
     parameters before rebuilding.
-    Requires the Jenkins :jenkins-wiki:`Rebuild Plugin <Rebuild+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Rebuild Plugin <rebuild>`.
 
     :arg bool auto-rebuild: Rebuild without asking for parameters
         (default false)
@@ -985,8 +1037,8 @@ def slave_prerequisites(registry, xml_parent, data):
     This plugin allows you to check prerequisites on slave before
     a job can run a build on it
 
-    Requires the Jenkins :jenkins-wiki:`Slave Prerequisites Plugin
-    <Slave+Prerequisites+Plugin>`.
+    Requires the Jenkins :jenkins-plugins:`Slave Prerequisites Plugin
+    <slave-prerequisites>`.
 
     :arg str script: A script to be executed on slave node.
         If returning non 0 status, the node will be vetoed from hosting
@@ -1024,8 +1076,8 @@ def groovy_label(registry, xml_parent, data):
     This plugin allows you to use Groovy script to restrict where this project
     can be run.
 
-    Requires the Jenkins :jenkins-wiki:`Groovy Label Assignment Plugin
-    <Groovy+Label+Assignment+plugin>`.
+    Requires the Jenkins :jenkins-plugins:`Groovy Label Assignment Plugin
+    <groovy-label-assignment>`.
 
     Return value from Groovy script is treated as Label Expression.
     It is treated as followings:
@@ -1078,8 +1130,8 @@ def groovy_label(registry, xml_parent, data):
 
 def lockable_resources(registry, xml_parent, data):
     """yaml: lockable-resources
-    Requires the Jenkins :jenkins-wiki:`Lockable Resources Plugin
-    <Lockable+Resources+Plugin>`.
+    Requires the Jenkins :jenkins-plugins:`Lockable Resources Plugin
+    <lockable-resources>`.
 
     :arg str resources: List of required resources, space separated.
         (required, mutual exclusive with label)
@@ -1138,7 +1190,7 @@ def lockable_resources(registry, xml_parent, data):
 
 def docker_container(registry, xml_parent, data):
     """yaml: docker-container
-    Requires the Jenkins: :jenkins-wiki:`Docker Plugin<Docker+Plugin>`.
+    Requires the Jenkins: :jenkins-plugins:`Docker Plugin <docker-plugin>`.
 
     :arg str docker-registry-url: URL of the Docker registry. (default '')
     :arg str credentials-id: Credentials Id for the Docker registey.
@@ -1190,8 +1242,9 @@ def docker_container(registry, xml_parent, data):
 def disable_resume(registry, xml_parent, data):
     """yaml: disable-resume
     Do not allow the pipeline to resume if the master restarts
-    Requires the Jenkins :jenkins-wiki:`Pipeline Job Plugin
-    <Pipeline+Job+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Pipeline Job Plugin
+    <workflow-aggregator>`.
 
     Example:
 
@@ -1211,8 +1264,8 @@ def cachet_gating(registry, xml_parent, data):
     The Cachet Gating Plugin provides a gating mechanism
     based on the availability of resources.
 
-    Requires the Jenkins: :jenkins-wiki:`Cachet Gate Plugin
-    <Cachet+Gate+Plugin>`.
+    Requires the Jenkins: :jenkins-plugins:`Cachet Gate Plugin
+    <cachet-gating>`.
 
     :arg bool required-resources: Confirm availability of listed
         resources before building. Requires the list of resources to
@@ -1239,6 +1292,136 @@ def cachet_gating(registry, xml_parent, data):
             XML.SubElement(resources, "string").text = str(resource)
 
 
+def office_365_connector(registry, xml_parent, data):
+    """yaml: office-365-connector
+    Used to send actionable messages to MS Outlook or Teams
+
+    Requires the Jenkins: :jenkins-plugins:` Office-365-Connector Plugin
+    <Office-365-Connector>`.
+
+    :arg list webhooks: List of webhooks (required)
+
+        * **url** (srt): URL generated in the Office 365 Connectors page (required)
+        * **name** (str): Allows to provide name fo the connection. Name is not
+            mandatory but helps managing when there are many connection
+            assigned to the build (optional, default '')
+        * **start-notification** (bool): If the notification should be sent on
+            start of build (optional, default False)
+        * **notify-success** (bool): If the notification should be sent on
+            succeeded build (optional, default True)
+        * **notify-aborted** (bool): If the notification should be sent on
+            aborted build (optional, default False)
+        * **notify-not-built** (bool): If the notification should be sent on
+            not built build (optional, default False)
+        * **notify-unstable** (bool): If the notification should be sent on
+            unstable build (optional, default True)
+        * **notify-failure** (bool): If the notification should be sent on
+            failed build (optional, default True)
+        * **notify-back-to-normal** (bool): If the notification should be sent on
+            back to normal build (optional, default True)
+        * **notify-repeated-failure** (bool): If the notification should be sent on
+            repeated failures (optional, default False)
+        * **timeout** (int): connection timeout (in milliseconds) for TCP and HTTP
+            (optional, default 30000)
+        * **macros** (list): List of macros
+
+            * **template** (str)
+              **value** (str)
+
+        * **fact-definitions** (list): List of fact definitions
+
+            * **name** (str)
+              **template** (str)
+
+    Example:
+
+    .. literalinclude:: /../../tests/properties/fixtures/office-365-connector-full.yaml
+        :language: yaml
+    """
+
+    office_365_connector = XML.SubElement(
+        xml_parent, "jenkins.plugins.office365connector.WebhookJobProperty"
+    )
+    office_365_connector.set("plugin", "Office-365-Connector")
+    webhooks = XML.SubElement(office_365_connector, "webhooks")
+
+    webhook_mapping = [
+        ("url", "url", None),
+        ("name", "name", ""),
+        ("start-notification", "startNotification", False),
+        ("notify-success", "notifySuccess", True),
+        ("notify-aborted", "notifyAborted", False),
+        ("notify-not-built", "notifyNotBuilt", False),
+        ("notify-unstable", "notifyUnstable", True),
+        ("notify-failure", "notifyFailure", True),
+        ("notify-back-to-normal", "notifyBackToNormal", True),
+        ("notify-repeated-failure", "notifyRepeatedFailure", False),
+        ("timeout", "timeout", 30000),
+    ]
+    macro_mapping = [("template", "template", None), ("value", "value", None)]
+    fact_definition_mapping = [("name", "name", None), ("template", "template", None)]
+
+    if "webhooks" not in data.keys():
+        raise MissingAttributeError("webhooks")
+
+    for webhook_data in data["webhooks"]:
+        webhook_element = XML.SubElement(
+            webhooks, "jenkins.plugins.office365connector.Webhook"
+        )
+        helpers.convert_mapping_to_xml(
+            webhook_element, webhook_data, webhook_mapping, fail_required=True
+        )
+        if "macros" in webhook_data.keys():
+            macros = XML.SubElement(webhook_element, "macros")
+            for macro_data in webhook_data["macros"]:
+                macro_element = XML.SubElement(
+                    macros, "jenkins.plugins.office365connector.model.Macro"
+                )
+                helpers.convert_mapping_to_xml(
+                    macro_element, macro_data, macro_mapping, fail_required=True
+                )
+        if "fact-definitions" in webhook_data.keys():
+            fact_definitions = XML.SubElement(webhook_element, "factDefinitions")
+            for fact_definition_data in webhook_data["fact-definitions"]:
+                fact_definition_element = XML.SubElement(
+                    fact_definitions,
+                    "jenkins.plugins.office365connector.model.FactDefinition",
+                )
+                helpers.convert_mapping_to_xml(
+                    fact_definition_element,
+                    fact_definition_data,
+                    fact_definition_mapping,
+                    fail_required=True,
+                )
+
+
+def speed_durability(registry, xml_parent, data):
+    """yaml: speed-durability
+    This setting allows users to change the default durability mode
+    for running Pipelines.
+
+    :arg str hint: speed durability hint to be used, can be performance-optimized,
+        survivable-non-atomic, max-survivability
+
+    Example:
+
+    .. literalinclude::
+        /../../tests/properties/fixtures/speed-durability.yaml
+       :language: yaml
+    """
+    dhp = XML.SubElement(
+        xml_parent,
+        "org.jenkinsci.plugins.workflow.job.properties.DurabilityHintJobProperty",
+    )
+    choicedict = {
+        "performance-optimized": "PERFORMANCE_OPTIMIZED",
+        "survivable-non-atomic": "SURVIVABLE_NONATOMIC",
+        "max-survivability": "MAX_SURVIVABILITY",
+    }
+    mapping = [("hint", "hint", None, choicedict)]
+    helpers.convert_mapping_to_xml(dhp, data, mapping, fail_required=True)
+
+
 class Properties(jenkins_jobs.modules.base.Base):
     sequence = 20
 
@@ -1255,11 +1438,11 @@ class Properties(jenkins_jobs.modules.base.Base):
             if next(iter(prop)) == "authorization":
                 # Only projects are placed in folders
                 if "project-type" in data:
-                    if data["project-type"] == "folder":
-                        prop["authorization"]["_use_folder_perms"] = True
+                    if data["project-type"] in ("folder", "multibranch"):
+                        prop["authorization"]["_is_a_folder"] = True
                     else:
-                        prop["authorization"]["_use_folder_perms"] = "folder" in data
+                        prop["authorization"]["_is_a_folder"] = False
                 else:
-                    prop["authorization"]["_use_folder_perms"] = False
+                    prop["authorization"]["_is_a_folder"] = False
 
             self.registry.dispatch("property", properties, prop)
