@@ -27,7 +27,7 @@ default with no SCM.
 
 The scm module allows referencing multiple repositories in a Jenkins job.
 Note: Adding more than one scm definition requires the Jenkins
-:jenkins-wiki:`Multiple SCMs plugin <Multiple+SCMs+Plugin>`.
+:jenkins-plugins:`Multiple SCMs plugin <multiple-scms>`.
 
 Example of multiple repositories in a single job:
     .. literalinclude:: /../../tests/macros/fixtures/scm/multi-scms001.yaml
@@ -47,8 +47,9 @@ import jenkins_jobs.modules.helpers as helpers
 
 def p4(registry, xml_parent, data):
     r"""yaml: p4
-    Specifies the Perforce (P4) repository for this job
-    Requires the Jenkins :jenkins-wiki:`P4 Plugin <P4+Plugin>`.
+    Specifies the Perforce (P4) repository for this job.
+
+    Requires the Jenkins :jenkins-plugins:`P4 Plugin <p4>`.
     """
     scm = XML.SubElement(
         xml_parent,
@@ -171,7 +172,8 @@ def p4_construct_populate(xml_parent, data):
 def git(registry, xml_parent, data):
     r"""yaml: git
     Specifies the git SCM repository for this job.
-    Requires the Jenkins :jenkins-wiki:`Git Plugin <Git+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Git Plugin <git>`.
 
     :arg str url: URL of the git repository
     :arg str credentials-id: ID of credential to use to connect, which is the
@@ -225,18 +227,18 @@ def git(registry, xml_parent, data):
             * **fisheye** - https://www.atlassian.com/software/fisheye
             * **gitblit** - http://gitblit.com/
             * **githubweb** - https://github.com/
-            * **gitiles** - https://code.google.com/p/gitiles/
+            * **gitiles** - https://code.google.com/archive/p/gitiles/
             * **gitlab** - https://about.gitlab.com/
             * **gitlist** - http://gitlist.org/
             * **gitoriousweb** - https://gitorious.org/
             * **gitweb** - https://git-scm.com/docs/gitweb
-            * **kiln** - https://www.fogcreek.com/kiln/
+            * **kiln** - https://www.fogbugz.com/version-control
             * **microsoft\-tfs\-2013** - |tfs_2013|
-            * **phabricator** - http://phabricator.org/
-            * **redmineweb** - http://www.redmine.org/
+            * **phabricator** - https://www.phacility.com/
+            * **redmineweb** - https://www.redmine.org/
             * **rhodecode** - https://rhodecode.com/
-            * **stash** - https://www.atlassian.com/software/bitbucket/server
-            * **viewgit** - http://viewgit.fealdia.org/
+            * **stash** - https://www.atlassian.com/software/bitbucket/enterprise/data-center
+            * **viewgit**
     :arg str browser-url: url for the repository browser (required if browser
         is not 'auto', no default)
     :arg str browser-version: version of the repository browser (GitLab only,
@@ -262,6 +264,9 @@ def git(registry, xml_parent, data):
         * **clean** (`dict`)
             * **after** (`bool`) - Clean the workspace after checkout
             * **before** (`bool`) - Clean the workspace before checkout
+        * **committer** (`dict`)
+            * **name** (`str`) - Name to use as author of new commits
+            * **email** (`str`) - E-mail address to use for new commits
         * **excluded-users**: (`list(string)`) - list of users to ignore
             revisions from when polling for changes.
             (if polling is enabled, optional)
@@ -297,6 +302,9 @@ def git(registry, xml_parent, data):
             (default false)
         * **honor-refspec** (`bool`) - Perform initial clone using the refspec
             defined for the repository (default false)
+        * **skip-notifications** (`bool`) - Skip build status notifications
+            (default false). Requires the Jenkins
+            :jenkins-plugins:`Skip Notifications Trait Plugin <skip-notifications-trait>`.
         * **sparse-checkout** (`dict`)
             * **paths** (`list`) - List of paths to sparse checkout. (optional)
         * **submodule** (`dict`)
@@ -326,7 +334,7 @@ def git(registry, xml_parent, data):
     .. literalinclude:: /../../tests/scm/fixtures/git001.yaml
 
     .. |tfs_2013| replace::
-        https://www.visualstudio.com/en-us/products/tfs-overview-vs.aspx
+        https://azure.microsoft.com/en-us/services/devops/server
 
     """
     logger = logging.getLogger("%s:git" % __name__)
@@ -522,6 +530,22 @@ def git_extensions(xml_parent, data):
                 ext = XML.SubElement(tr, "extension", {"class": ext_name})
             else:
                 ext = XML.SubElement(xml_parent, ext_name)
+    committer = data.get("committer", {})
+    if committer:
+        ext_name = impl_prefix + "UserIdentity"
+        if trait:
+            trait_name = "com.cloudbees.jenkins.plugins.bitbucket.notifications.SkipNotificationsTrait"
+            trait_name = "UserIdentityTrait"
+            tr = XML.SubElement(xml_parent, trait_prefix + trait_name)
+            ext = XML.SubElement(tr, "extension", {"class": ext_name})
+        else:
+            ext = XML.SubElement(xml_parent, ext_name)
+        name = committer.get("name")
+        if name:
+            XML.SubElement(ext, "name").text = name
+        email = committer.get("email")
+        if email:
+            XML.SubElement(ext, "email").text = email
     if not trait and "excluded-users" in data:
         excluded_users = "\n".join(data["excluded-users"])
         ext = XML.SubElement(xml_parent, impl_prefix + "UserExclusion")
@@ -605,6 +629,10 @@ def git_extensions(xml_parent, data):
             ).lower()
         if "reference-repo" in data:
             XML.SubElement(ext, "reference").text = str(data["reference-repo"])
+    skip_notifications = data.get("skip-notifications", False)
+    if trait and skip_notifications:
+        trait_name = "com.cloudbees.jenkins.plugins.bitbucket.notifications.SkipNotificationsTrait"
+        XML.SubElement(xml_parent, trait_name)
     if not trait and "sparse-checkout" in data:
         ext_name = impl_prefix + "SparseCheckoutPaths"
         ext = XML.SubElement(xml_parent, ext_name)
@@ -704,7 +732,8 @@ def git_extensions(xml_parent, data):
 def cvs(registry, xml_parent, data):
     """yaml: cvs
     Specifies the CVS SCM repository for this job.
-    Requires the Jenkins :jenkins-wiki:`CVS Plugin <CVS+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`CVS Plugin <cvs>`.
 
     :arg list repos: List of CVS repositories. (required)
 
@@ -835,7 +864,8 @@ def cvs(registry, xml_parent, data):
 def repo(registry, xml_parent, data):
     """yaml: repo
     Specifies the repo SCM repository for this job.
-    Requires the Jenkins :jenkins-wiki:`Repo Plugin <Repo+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Repo Plugin <repo>`.
 
     :arg str manifest-url: URL of the repo manifest (required)
     :arg str manifest-branch: The branch of the manifest to use (optional)
@@ -916,8 +946,9 @@ def repo(registry, xml_parent, data):
 def store(registry, xml_parent, data):
     """yaml: store
     Specifies the Visualworks Smalltalk Store repository for this job.
-    Requires the Jenkins :jenkins-wiki:`Visualworks Smalltalk Store Plugin
-    <Visualworks+Smalltalk+Store+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Visualworks Smalltalk Store Plugin
+    <visualworks-store>`.
 
     :arg str script: name of the Store script to run
     :arg str repository: name of the Store repository
@@ -1121,8 +1152,9 @@ def svn(registry, xml_parent, data):
 def tfs(registry, xml_parent, data):
     r"""yaml: tfs
     Specifies the Team Foundation Server repository for this job.
-    Requires the Jenkins :jenkins-wiki:`Team Foundation Server Plugin
-    <Team+Foundation+Server+Plugin>`.
+
+    Requires the Jenkins Team Foundation Server Plugin
+    (https://github.com/jenkinsci/tfs-plugin).
 
     **NOTE**: TFS Password must be entered manually on the project if a
     user name is specified. The password will be overwritten with an empty
@@ -1225,8 +1257,9 @@ def tfs(registry, xml_parent, data):
 def workspace(registry, xml_parent, data):
     """yaml: workspace
     Specifies the cloned workspace for this job to use as a SCM source.
-    Requires the Jenkins :jenkins-wiki:`Clone Workspace SCM Plugin
-    <Clone+Workspace+SCM+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Clone Workspace SCM Plugin
+    <clone-workspace-scm>`.
 
     The job the workspace is cloned from must be configured with an
     clone-workspace publisher
@@ -1262,7 +1295,8 @@ def workspace(registry, xml_parent, data):
 def hg(self, xml_parent, data):
     """yaml: hg
     Specifies the mercurial SCM repository for this job.
-    Requires the Jenkins :jenkins-wiki:`Mercurial Plugin <Mercurial+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Mercurial Plugin <mercurial>`.
 
     :arg str url: URL of the hg repository (required)
     :arg str credentials-id: ID of credentials to use to connect (optional)
@@ -1285,8 +1319,8 @@ def hg(self, xml_parent, data):
             * **bitbucketweb** - https://bitbucket.org/
             * **fisheye** - https://www.atlassian.com/software/fisheye
             * **googlecode** - https://code.google.com/
-            * **hgweb** - https://www.selenic.com/hg/help/hgweb
-            * **kilnhg** - https://www.fogcreek.com/kiln/
+            * **hgweb** - https://www.mercurial-scm.org/repo/hg/help/hgweb
+            * **kilnhg** - https://www.fogbugz.com/version-control
             * **rhodecode** - https://rhodecode.com/ (versions >= 1.2)
             * **rhodecode-pre-1.2.0** - https://rhodecode.com/ (versions < 1.2)
 
@@ -1354,8 +1388,9 @@ def openshift_img_streams(registry, xml_parent, data):
     OpenShift ImageStreams (which are abstractions of Docker repositories)
     and SCMs - versions / commit IDs of related artifacts
     (images vs. programmatics files)
-    Requires the Jenkins :jenkins-wiki:`OpenShift
-    Pipeline Plugin <OpenShift+Pipeline+Plugin>`._
+
+    Requires the Jenkins :jenkins-plugins:`OpenShift Pipeline Plugin
+    <openshift-pipeline>`.
 
     :arg str image-stream-name: The name of the ImageStream is what shows up
         in the NAME column if you dump all the ImageStream's with the
@@ -1405,7 +1440,8 @@ def openshift_img_streams(registry, xml_parent, data):
 def bzr(registry, xml_parent, data):
     """yaml: bzr
     Specifies the bzr SCM repository for this job.
-    Requires the Jenkins :jenkins-wiki:`Bazaar Plugin <Bazaar+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`Bazaar Plugin <bazaar>`.
 
     :arg str url: URL of the bzr branch (required)
     :arg bool clean-tree: Clean up the workspace (using bzr) before pulling
@@ -1470,7 +1506,8 @@ def url(registry, xml_parent, data):
     """yaml: url
 
     Watch for changes in, and download an artifact from a particular url.
-    Requires the Jenkins :jenkins-wiki:`URL SCM <URL+SCM>`.
+
+    Requires the Jenkins :jenkins-plugins:`URL SCM <URLSCM>`.
 
     :arg list url-list: List of URLs to watch. (required)
     :arg bool clear-workspace: If set to true, clear the workspace before
@@ -1498,7 +1535,8 @@ def dimensions(registry, xml_parent, data):
     """yaml: dimensions
 
     Specifies the Dimensions SCM repository for this job.
-    Requires Jenkins :jenkins-wiki:`Dimensions Plugin <Dimensions+Plugin>`.
+
+    Requires Jenkins :jenkins-plugins:`Dimensions Plugin <dimensionsscm>`.
 
     :arg str project: Project name of format PRODUCT_ID:PROJECT_NAME (required)
     :arg str permissions: Default Permissions for updated files
@@ -1600,7 +1638,8 @@ def dimensions(registry, xml_parent, data):
 def accurev(registry, xml_parent, data):
     """yaml: accurev
     Specifies the AccuRev SCM repository for this job.
-    Requires the Jenkins :jenkins-wiki:`AccuRev Plugin <AccuRev+Plugin>`.
+
+    Requires the Jenkins :jenkins-plugins:`AccuRev Plugin <accurev>`.
 
     :arg str depot: Depot you want to use for the current job (optional)
     :arg str stream: Stream where the build will be generated from (optional)
@@ -1666,8 +1705,14 @@ class SCM(jenkins_jobs.modules.base.Base):
     def gen_xml(self, xml_parent, data):
 
         # multibranch-pipeline scm implementation is incompatible with SCM
-        if data.get("project-type") in ["multibranch", "multibranch-defaults"]:
-            logging.debug("SCM Module skipped for multibranch project-type.")
+        if data.get("project-type") in [
+            "multibranch",
+            "multibranch-defaults",
+            "pipeline",
+        ]:
+            logging.debug(
+                "SCM Module skipped for %s project-type." % data.get("project-type")
+            )
             return
 
         scms_parent = XML.Element("scms")
